<script setup lang="ts">
import { computed, ref } from 'vue';

import { useVbenDrawer, useVbenModal } from '@vben/common-ui';
import { $t } from '@vben/locales';
import { addFullName, cloneDeep, getPopupContainer } from '@vben/utils';

import { InputSearch, Modal } from 'ant-design-vue';

import { useVbenForm } from '#/adapter/form';
import { addRepairRes, getRepairRes, updateRepairRes } from '#/api/eims/repair-res';
import { getDeptTree, userList } from '#/api/system/user';
import reqModal from '#/views/eims/components/repair-req-modal.vue';

import { drawerSchema } from './data';

const emit = defineEmits<{ reload: [] }>();

const isUpdate = ref(false);
const title = computed(() => {
  return isUpdate.value ? $t('pages.common.edit') : $t('pages.common.add');
});

const [BasicForm, formApi] = useVbenForm({
  commonConfig: {
    formItemClass: 'col-span-2',
    componentProps: {
      class: 'w-full'
    },
    labelWidth: 120
  },
  schema: drawerSchema(),
  showDefaultActions: false,
  wrapperClass: 'grid-cols-2'
});

const [BasicDrawer, drawerApi] = useVbenDrawer({
  onCancel: handleCancel,
  onConfirm: handleConfirm,
  async onOpenChange(isOpen) {
    if (!isOpen) {
      return null;
    }

    drawerApi.drawerLoading(true);
    const { id } = drawerApi.getData() as { id?: number | string };
    const { resUser } = drawerApi.getData() as { resUser?: number | string };
    const { resDept } = drawerApi.getData() as { resDept?: number | string };
    isUpdate.value = !!id;
    // 初始化
    await setupDeptSelect();
    // 新增时默认填充登录人为报修人
    if (!isUpdate.value && resUser !== null && resDept !== null) {
      await formApi.setValues({ 'resUser': resUser, 'resDept': resDept });
      await setupUserOptions(resDept);
    }
    formApi.updateSchema([
      {
        componentProps: {
          disabled: isUpdate.value
        },
        fieldName: 'resCode'
      }
    ]);
    // 更新 && 赋值
    if (isUpdate.value && id) {
      const record = await getRepairRes(id);
      await formApi.setValues(record);
      if (isUpdate.value && record.resDept) {
        await setupUserOptions(record.resDept);
      }
    }

    drawerApi.drawerLoading(false);
  }
});

const [ReqModal, reqModalApi] = useVbenModal({
  connectedComponent: reqModal,
  draggable: true,
  title: '选择报修单'
});

function handleReqModal() {
  reqModalApi.setData({});
  reqModalApi.open();
}
/**
 * 用户的加载
 */
async function setupUserOptions(deptId: any) {
  const params = { deptId };
  const userPageResult = await userList({
    pageNum: 1,
    pageSize: 500,
    ...params
  });
  const options = userPageResult.rows.map((item) => ({
    label: item.nickName || item.userName,
    value: item.userId
  }));
  // 筛选
  const filterOption = (input: string, option: any) => {
    return option.label.toLowerCase().includes(input.toLowerCase());
  };

  const placeholder = options.length > 0 ? '请选择' : '该部门下暂无用户';
  formApi.updateSchema([
    {
      componentProps: { options, placeholder, filterOption },
      fieldName: 'resUser'
    }
  ]);
}

/**
 * 初始化部门选择
 */
async function setupDeptSelect() {
  // updateSchema
  const deptTree = await getDeptTree();
  // 选中后显示在输入框的值 即父节点 / 子节点
  addFullName(deptTree, 'label', ' / ');
  formApi.updateSchema([
    {
      componentProps: (formModel) => ({
        class: 'w-full',
        fieldNames: {
          key: 'id',
          value: 'id',
          children: 'children'
        },
        getPopupContainer,
        async onSelect(deptId: number | string) {
          /** 根据部门ID加载用户 */
          await setupUserOptions(deptId);
          /** 变化后需要重新选择用户 */
          formModel.resUser = undefined;
        },
        placeholder: '请选择',
        showSearch: true,
        treeData: deptTree,
        treeDefaultExpandAll: true,
        treeLine: { showLeafIcon: false },
        // 筛选的字段
        treeNodeFilterProp: 'label',
        // 选中后显示在输入框的值
        treeNodeLabelProp: 'fullName'
      }),
      fieldName: 'resDept'
    }
  ]);
}

async function handleConfirm() {
  try {
    drawerApi.drawerLoading(true);
    const { valid } = await formApi.validate();
    if (!valid) {
      return;
    }
    const data = cloneDeep(await formApi.getValues());
    await (isUpdate.value ? updateRepairRes(data) : addRepairRes(data));
    emit('reload');
    await handleCancel();
  } catch (error) {
    console.error(error);
  } finally {
    drawerApi.drawerLoading(false);
  }
}

async function handleCancel() {
  drawerApi.close();
  await formApi.resetForm();
}

/**
 * 打开查询报修单
 */
function onSearchReq() {
  handleReqModal();
}
/**
 * 更新选择的报修单
 * @param req
 */
async function updateSelect(req: any) {
  // 选择报修单后生成维修单号
  if (!req.code || !req.id) {
    Modal.error({
      content: '获取保修单失败，请重新登录后重试！',
      title: '提示'
    });
  }
  const resCode = `WXD${req.code.slice(3)}`;
  await formApi.setValues({ 'reqId': req.id, 'reqCode': req.code, 'resCode': resCode });
}
</script>

<template>
  <BasicDrawer :close-on-click-modal="false" :title="title" class="w-[600px]">
    <BasicForm>
      <template #reqCode="slotProps">
        <InputSearch :enter-button="true" placeholder="请选择" @search="onSearchReq" v-bind="slotProps" :disabled="isUpdate" />
      </template>
    </BasicForm>
    <ReqModal class="w-[1200px]" @update-select="updateSelect" />
  </BasicDrawer>
</template>
